//+------------------------------------------------------------------+
//|                                                QuoteReceiver.mq4 |
//|                                       Copyright  2014, tidicofx |
//|                                               tidicofx@yahoo.com |
//|                                         developed for caspermhlo |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, tidicofx"
#property link      "tidicofx@yahoo.com"
#property version "1.0"
#property strict
//+------------------------------------------------------------------+
#include <stdlib.mqh>
#include <stderror.mqh>
//+------------------------------------------------------------------+
#import "MT4iQuickChannel.dll"
   int QC_StartReceiverW(string, int);
   int QC_ReleaseReceiver(int);
   int QC_GetMessages5W(int, uchar&[], int);
#import
#define QC_BUFFER_SIZE     10000
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Input Parameters Definition                                      |
//+------------------------------------------------------------------+

input string section0 = "====== general setting ======";
input string Group = "pipe";

input string section1 = "====== money management setting ======";
input bool   AutoLot = false;
input double Risk = 1;
input double FixedLot = 0.01;
input double StopTradeOnDrawDown = 10;

input string section2 = "====== entry exit setting ======";
input int    MaxOrders = 5;
input double EntryGapTrigger = 2;
input double ExitGapTrigger = 0.1;
input double MaxSpread = 2.5;
input double SL = 10, TP = 10;
input bool   CloseOnlyInProfit = true;
input int    TradeMinOpenTime = 0;
input int    TradeMaxOpenTime = 0;

input string section3 = "====== trail setting ======";
input bool   Trailing = false;
input double TrailStart = 5, TrailStop = 5, TrailStep = 1;

input string section4 = "====== timing setting ======";
input bool   LocalTime = false;
input string TradingDays = "1;2;3;4;5";
input string TradingDaysNote = "0-Sunday,1,2,3,4,5,6";
input string StartHour1 = "00:00", StopHour1 = "24:00";
input string StartHour2 = "00:00", StopHour2 = "00:00";
input string StartHour3 = "00:00", StopHour3 = "00:00";

input string section5 = "====== other settings ======";
input string EAName = "Monster EA";
input string TradeComment = "";
input int    MagicNumber = 200;

input string section6 = "====== display settings ======";
input bool   Display = true;
input string FontName = "Tahoma";
input int    FontSize = 9;
input int    TextCorner = 1;
input color  TextColor = clrGold;
input int    TextXAnchor = 15;
input int    TextYAnchor = 20;
//+------------------------------------------------------------------+
//| Local Parameters Definition                                      |
//+------------------------------------------------------------------+
string lbl = "QuoteReceiver.", gbl;
string ver = "1.0.0";
string version_info;
int DecimalPoints, pipe = 0;
double sAsk, sBid;
int LastTrade;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
  //------------------------------------------------------------------
  DecimalPoints = 1;
  double x = MarketInfo(Symbol(), MODE_LOTSTEP) * 10;
  while (x < 1)
  { x *= 10; DecimalPoints += 1; }
  //------------------------------------------------------------------
  string msg;
  if (!IsExpertEnabled()) { msg = "experts advisors is not enabled...\n ea is stopped working"; Alert(msg); Comment(msg); return (INIT_FAILED); }
  if (!IsDllsAllowed()) { msg = "DLL is not enabled...\n ea is stopped working"; Alert(msg); Comment(msg); return (INIT_FAILED); }
  //------------------------------------------------------------------
  gbl = lbl + Symbol() + "." + IntegerToString(MagicNumber) + ".";
  LastTrade = -1;
  //------------------------------------------------------------------
  return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
  //------------------------------------------------------------------
  if (reason == REASON_REMOVE) GlobalVariablesDeleteAll(gbl);
  if (Display) RemoveInfo();
  Comment("");
  if (pipe > 0) QC_ReleaseReceiver(pipe);
  sAsk = 0; sBid = 0;
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
void OnTick()
{
  //------------------------------------------------------------------
  bool AllowTrade = true;
  double Equity = GlobalVariableGet(gbl+"Equity");
  if (Equity == 0) { Equity = AccountEquity(); GlobalVariableSet(gbl+"Equity", AccountEquity()); }
  if (NormalizeDouble(AccountEquity()/Equity-StopTradeOnDrawDown/100, 4)<=0)
  {
    AllowTrade = false;
    Comment("ea stopped because of max drawdown reached. (DrawDown="+DoubleToStr(100-Equity/AccountEquity()*100, 2)+"%)");
  }
  else Comment("");
  //------------------------------------------------------------------
  if (AllowTrade && StringFind(TradingDays, DoubleToStr(DayOfWeek(), 0)) == -1) AllowTrade = false;
  datetime dt = TimeCurrent(); if (LocalTime) dt = TimeLocal();
  if (AllowTrade)
  {
    if (StartHour1 < StopHour1)
      if (TimeToStr(dt, TIME_MINUTES) < StartHour1 || TimeToStr(dt, TIME_MINUTES) > StopHour1)
        AllowTrade = false;
    if (StartHour1 > StopHour1)
      if (TimeToStr(dt, TIME_MINUTES) < StartHour1 && TimeToStr(dt, TIME_MINUTES) > StopHour1)
        AllowTrade = false;
  }
  if (AllowTrade)
  {
    if (StartHour2 < StopHour2)
      if (TimeToStr(dt, TIME_MINUTES) < StartHour2 || TimeToStr(dt, TIME_MINUTES) > StopHour2)
        AllowTrade = false;
    if (StartHour2 > StopHour2)
      if (TimeToStr(dt, TIME_MINUTES) < StartHour2 && TimeToStr(dt, TIME_MINUTES) > StopHour2)
        AllowTrade = false;
  }
  if (AllowTrade)
  {
    if (StartHour3 < StopHour3)
      if (TimeToStr(dt, TIME_MINUTES) < StartHour3 || TimeToStr(dt, TIME_MINUTES) > StopHour3)
        AllowTrade = false;
    if (StartHour3 > StopHour3)
      if (TimeToStr(dt, TIME_MINUTES) < StartHour3 && TimeToStr(dt, TIME_MINUTES) > StopHour3)
        AllowTrade = false;
  }
  if (AllowTrade)
    if (GetOrderCount() >= MaxOrders) AllowTrade = false;
  if (AllowTrade)
    if (MaxSpread != 0 && NormalizeDouble(Ask-Bid-PipsToPoints(MaxSpread), _Digits)>0) AllowTrade = false;
  if (AllowTrade)
    if (MaxSpread != 0 && NormalizeDouble(Ask-Bid-PipsToPoints(MaxSpread), _Digits)>0) AllowTrade = false;
  //------------------------------------------------------------------
  RefreshRates();
  string arrMessages[] = {};
  int MessagesRetrieved = CheckForMessages(arrMessages);
  int i = ArraySize(arrMessages)-1;
  sAsk = 0; sBid = 0;
  if (i >= 0)
  {
    string sp[]; StringSplit2(arrMessages[i], sp, "|");
    sAsk = StrToDouble(sp[0]); sBid = StrToDouble(sp[1]);
    double gap = (Ask+Bid)/2-(sAsk+sBid)/2;
    if (LastTrade == OP_BUY && NormalizeDouble(gap-PipsToPoints(ExitGapTrigger), _Digits)<=0) LastTrade = -1;
    if (LastTrade == OP_SELL && NormalizeDouble(-gap-PipsToPoints(ExitGapTrigger), _Digits)<=0) LastTrade = -1;
    if (NormalizeDouble(MathAbs(gap)-PipsToPoints(ExitGapTrigger), _Digits)>=0)
    if (sAsk != 0 && sBid != 0)
    {
      if (AllowTrade && LastTrade != OP_BUY && NormalizeDouble(PointsToPips(gap)-EntryGapTrigger*PointsToPips(Ask-Bid), _Digits)>=0)
      {
        Print("try to enter long trade (gap = "+DoubleToStr(PointsToPips(gap), 1)+", trigger="+DoubleToStr(PointsToPips(EntryGapTrigger*(Ask-Bid)), 1));
        SetOrder(OP_BUY);
      }
      if (AllowTrade && LastTrade != OP_SELL && NormalizeDouble(PointsToPips(-gap)-EntryGapTrigger*PointsToPips(Ask-Bid), _Digits)>=0)
      {
        Print("try to enter short trade (gap = "+DoubleToStr(PointsToPips(gap), 1)+", trigger="+DoubleToStr(PointsToPips(EntryGapTrigger*(Ask-Bid)), 1));
        SetOrder(OP_SELL);
      }
      for (i = OrdersTotal() - 1; i >= 0; i--)
        if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
          if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
          {
            color col = clrBlack;
            if (OrderType() == OP_BUY) col = clrBlue;
            if (OrderType() == OP_SELL) col = clrRed;
            if (TradeMaxOpenTime != 0 && TimeCurrent()-OrderOpenTime()>=TradeMaxOpenTime)
              if (OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 0, col))
                continue;
            if (TradeMinOpenTime != 0 && TimeCurrent()-OrderOpenTime()<TradeMinOpenTime) continue;
            if ((!CloseOnlyInProfit || NormalizeDouble(OrderProfit()+OrderCommission()+OrderSwap(), 4)>=0) && OrderType() == OP_BUY && NormalizeDouble(gap-PipsToPoints(ExitGapTrigger), _Digits)<=0)
              if (OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 0, col))
                continue;
            if ((!CloseOnlyInProfit || NormalizeDouble(OrderProfit()+OrderCommission()+OrderSwap(), 4)>=0) && OrderType() == OP_SELL && NormalizeDouble(-gap-PipsToPoints(ExitGapTrigger), _Digits)<=0)
              if (OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 0, col))
                continue;
          }
    }
  }
  //------------------------------------------------------------------
  for (i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {
        color col = clrBlack;
        if (OrderType() == OP_BUY) col = clrBlue;
        if (OrderType() == OP_SELL) col = clrRed;
        if (TradeMaxOpenTime != 0 && TimeCurrent()-OrderOpenTime()>=TradeMaxOpenTime)
          if (OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 0, col))
            continue;
      }
  //------------------------------------------------------------------
  if (Trailing) DoTrail();
  if (Display) DisplayInfo();
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
int SetOrder(int type)
{
  double price = 0;
  color col = clrBlack;
  if (type == OP_BUY) { col = clrBlue; price = Ask; }
  if (type == OP_SELL) { col = clrRed; price = Bid; }
  double lot = FixedLot;
  if (AutoLot)
  {
    double PipValue = MarketInfo(Symbol(), MODE_TICKVALUE) / MarketInfo(Symbol(), MODE_TICKSIZE);
    lot = NormalizeDouble(AccountEquity() * Risk / (100 * PipsToPoints(SL) * PipValue), DecimalPoints);
  }
  if (lot < MarketInfo(Symbol(), MODE_MINLOT)) lot = MarketInfo(Symbol(), MODE_MINLOT);
  if (lot > MarketInfo(Symbol(), MODE_MAXLOT)) lot = MarketInfo(Symbol(), MODE_MAXLOT);
  int ticket = OrderSend(Symbol(), type, lot, price, 0, 0, 0, TradeComment, MagicNumber, 0, col);
  int err = GetLastError();
  if (ticket == -1 && err > 0)
    Print("err=", IntegerToString(err), ", type=" + IntegerToString(type), ", price=", DoubleToStr(price, _Digits), ", lot=", DoubleToStr(lot, DecimalPoints), ", ", ErrorDescription(err));
  else if (ticket != -1) { LastTrade = type; SetSLTP(); }
  return (ticket);
}
//+------------------------------------------------------------------+
void SetSLTP()
{
  double sl, tp;
  for (int i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {
        if (OrderStopLoss() == 0 && OrderTakeProfit() == 0)
        {
          sl = 0; tp = 0;
          if (OrderType() == OP_BUY)
          {
            if (SL != 0) sl = OrderOpenPrice() - PipsToPoints(SL);
            if (TP != 0) tp = OrderOpenPrice() + PipsToPoints(TP);
          }
          if (OrderType() == OP_SELL)
          {
            if (SL != 0) sl = OrderOpenPrice() + PipsToPoints(SL);
            if (TP != 0) tp = OrderOpenPrice() - PipsToPoints(TP);
          }
          if (sl != 0 || tp != 0) OrderModify(OrderTicket(), OrderOpenPrice(), sl, tp, 0);
        }
      }
}
//+------------------------------------------------------------------+
void DoTrail()
{
  double sl;
  int step;
  for (int i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {
        sl = OrderStopLoss();
        if (sl == 0) sl = OrderOpenPrice();
        if (OrderType() == OP_BUY && NormalizeDouble(Bid-OrderOpenPrice(), Digits)>=NormalizeDouble(PipsToPoints(TrailStart), Digits))
        {
          step = (int)(MathFloor((Bid-sl) / PipsToPoints(TrailStep)));
          if (step >= 1 && (OrderStopLoss() == 0 || NormalizeDouble(OrderStopLoss(), Digits)<NormalizeDouble(Bid-PipsToPoints(TrailStop), Digits)))
            OrderModify(OrderTicket(), OrderOpenPrice(), NormalizeDouble(Bid-PipsToPoints(TrailStop), Digits), OrderTakeProfit(), 0);
        }
        if (OrderType() == OP_SELL && NormalizeDouble(OrderOpenPrice()-Ask, Digits)>=NormalizeDouble(PipsToPoints(TrailStart), Digits))
        {
          step = (int)(MathFloor((sl-Ask) / PipsToPoints(TrailStep)));
          if (step >= 1 && (OrderStopLoss() == 0 || NormalizeDouble(OrderStopLoss(), Digits)>NormalizeDouble(Ask+PipsToPoints(TrailStop), Digits)))
            OrderModify(OrderTicket(), OrderOpenPrice(), NormalizeDouble(Ask+PipsToPoints(TrailStop), Digits), OrderTakeProfit(), 0);
        }
      }
}
//+------------------------------------------------------------------+
int GetOrderCount()
{
  int count = 0;
  for (int i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
        count++;
  return (count);
}
//+------------------------------------------------------------------+
int CheckForMessages(string &messages[])
{
  if (pipe <= 0) pipe = QC_StartReceiverW(Group, WindowHandle(Symbol(), Period()));
  if (pipe <= 0) return (0);
  int MessagesFound = 0;
  uchar buffer[];
  ArrayResize(buffer, QC_BUFFER_SIZE);
  int res = QC_GetMessages5W(pipe, buffer, QC_BUFFER_SIZE);
  if (res > 0)
  {
    string strMsg = CharArrayToString(buffer, 0, res);
    string splitted[]; StringSplit(strMsg, '\t', splitted);
    for (int i = 0; i < ArraySize(splitted); i++)
    {
      if (StringTrimRight(StringTrimLeft(splitted[i])) != "")
      {
        MessagesFound++; ArrayResize(messages, MessagesFound);
        messages[MessagesFound-1] = splitted[i];
      }
    }
  }
  return (MessagesFound);
}
//+------------------------------------------------------------------+
void DisplayInfo()
{
  //------------------------------------------------------------------
  int index = 1;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Welcome to "+EAName, FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Broker: "+AccountCompany(), FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Account: "+IntegerToString(AccountNumber()), FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Balance: "+DoubleToStr(AccountBalance(), 2), FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Equity: "+DoubleToStr(AccountEquity(), 2), FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Max Spread: "+DoubleToStr(MaxSpread, 1)+"Pips", FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Spread: "+DoubleToStr(PointsToPips(Ask-Bid), 1)+"Pips", FontSize, FontName);
  index++;
  //------------------------------------------------------------------
  double a, b;
  if (sAsk == 0) a = Ask; else a = sAsk; if (sBid == 0) b = Bid; else b = sBid;
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Sender Ask: "+DoubleToStr(sAsk, _Digits), FontSize, FontName);
  index++;
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Sender Bid: "+DoubleToStr(sBid, _Digits), FontSize, FontName);
  index++;
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Ask Gap: "+DoubleToStr(PointsToPips(Ask-a), 1)+"Pips", FontSize, FontName);
  index++;
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Bid Gap: "+DoubleToStr(PointsToPips(Bid-b), 1)+"Pips", FontSize, FontName);
  index++;
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Average Gap: "+DoubleToStr(PointsToPips(MathAbs((Ask+Bid)/2-(a+b)/2)), 1)+"Pips", FontSize, FontName);
  index++;
  DrawLabel(lbl+IntegerToString(index), TextXAnchor, TextYAnchor * index, TextColor);
  ObjectSetText(lbl+IntegerToString(index), "Entry Trigger: "+DoubleToStr(PointsToPips(EntryGapTrigger*(Ask-Bid)), 1)+"Pips", FontSize, FontName);
  index++;
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
void RemoveInfo()
{
  for (int i = 1; i < 30; i++)
    if (ObjectFind(lbl+IntegerToString(i)) != -1) ObjectDelete(lbl+IntegerToString(i));
}
//+------------------------------------------------------------------+
void StringSplit2(string InputString, string &ResultArray[], string Separator)
{
  ArrayResize(ResultArray, 0);
  int lenSeparator = StringLen(Separator), NewArraySize;
  while (InputString != "") {
    int p = StringFind(InputString, Separator);
    if (p == -1) {
      NewArraySize = ArraySize(ResultArray) + 1;
      ArrayResize(ResultArray, NewArraySize);      
      ResultArray[NewArraySize - 1] = InputString;
      InputString = "";
    } else {
      NewArraySize = ArraySize(ResultArray) + 1;
      ArrayResize(ResultArray, NewArraySize);      
      ResultArray[NewArraySize - 1] = StringSubstr(InputString, 0, p);
      InputString = StringSubstr(InputString, p + lenSeparator);
      if (InputString == "") {
         ArrayResize(ResultArray, NewArraySize + 1);      
         ResultArray[NewArraySize] = "";
      }
    }     
  }
}
//+------------------------------------------------------------------+
void DrawLabel(string name, int x, int y, color col)
{
  if (ObjectFind(name) == -1)
  {
    ObjectCreate(name, OBJ_LABEL, 0, 0, 0);
    ObjectSet(name, OBJPROP_BACK, false);
    ObjectSet(name, OBJPROP_COLOR, col);
    ObjectSet(name, OBJPROP_CORNER, TextCorner);
    ObjectSet(name, OBJPROP_XDISTANCE, x);
    ObjectSet(name, OBJPROP_YDISTANCE, y);
  }
}
//+------------------------------------------------------------------+
double PipsToPoints(double pips)
{
  if (Digits == 5 || Digits == 3) pips *= 10;
  return (pips * Point);
}
//+------------------------------------------------------------------+
double PointsToPips(double points)
{
  if (_Digits == 5 || _Digits == 3) points /= 10;
  return (points / Point);
}
//+------------------------------------------------------------------+
string GetOrderTypeName(int type)
{
  if (type == OP_BUY || type == OP_BUYSTOP) return ("Buy");
  else if (type == OP_SELL || type == OP_SELLSTOP) return ("Sell");
  else return("None");
}
//+------------------------------------------------------------------+
string GetTimeFrameName(int TimeFrame)
{
  switch (TimeFrame)
  {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    case 0: return(GetTimeFrameName(Period()));
  }
  return ("");
}
//+------------------------------------------------------------------+
